==> vala-0.1.6.tar.gz: vala-0.1.6/ccode/valaccodefunctioncall.c <==
/* valaccodefunctioncall.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <ccode/valaccodefunctioncall.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <gee/readonlylist.h>
#include <stdlib.h>
#include <string.h>
#include <ccode/valaccodenode.h>

struct _ValaCCodeFunctionCallPrivate {
	ValaCCodeExpression* _call;
	GeeList* arguments;
};
#define VALA_CCODE_FUNCTION_CALL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_FUNCTION_CALL, ValaCCodeFunctionCallPrivate))
enum  {
	VALA_CCODE_FUNCTION_CALL_DUMMY_PROPERTY,
	VALA_CCODE_FUNCTION_CALL_CALL
};
static void vala_ccode_function_call_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static gpointer vala_ccode_function_call_parent_class = NULL;
static void vala_ccode_function_call_dispose (GObject * obj);


ValaCCodeFunctionCall* vala_ccode_function_call_new (ValaCCodeExpression* call) {
	ValaCCodeFunctionCall * self;
	g_return_val_if_fail (call == NULL || VALA_IS_CCODE_EXPRESSION (call), NULL);
	self = g_object_newv (VALA_TYPE_CCODE_FUNCTION_CALL, 0, NULL);
	vala_ccode_function_call_set_call (self, call);
	return self;
}


/**
 * Appends the specified expression to the list of arguments.
 *
 * @param expr a C code expression
 */
void vala_ccode_function_call_add_argument (ValaCCodeFunctionCall* self, ValaCCodeExpression* expr) {
	g_return_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self));
	g_return_if_fail (expr == NULL || VALA_IS_CCODE_EXPRESSION (expr));
	gee_collection_add (GEE_COLLECTION (self->priv->arguments), expr);
}


/**
 * Returns a copy of the list of arguments.
 *
 * @return list of arguments
 */
GeeList* vala_ccode_function_call_get_arguments (ValaCCodeFunctionCall* self) {
	g_return_val_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self), NULL);
	return GEE_LIST (gee_read_only_list_new (VALA_TYPE_CCODE_EXPRESSION, ((GBoxedCopyFunc) g_object_ref), g_object_unref, self->priv->arguments));
}


static void vala_ccode_function_call_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeFunctionCall * self;
	gboolean first;
	self = VALA_CCODE_FUNCTION_CALL (base);
	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_function_call_get_call (self)), writer);
	vala_ccode_writer_write_string (writer, " (");
	first = TRUE;
	{
		GeeList* expr_collection;
		GeeIterator* expr_it;
		expr_collection = self->priv->arguments;
		expr_it = gee_iterable_iterator (GEE_ITERABLE (expr_collection));
		while (gee_iterator_next (expr_it)) {
			ValaCCodeExpression* expr;
			expr = gee_iterator_get (expr_it);
			{
				if (!first) {
					vala_ccode_writer_write_string (writer, ", ");
				} else {
					first = FALSE;
				}
				if (expr != NULL) {
					vala_ccode_node_write (VALA_CCODE_NODE (expr), writer);
				}
				(expr == NULL ? NULL : (expr = (g_object_unref (expr), NULL)));
			}
		}
		(expr_it == NULL ? NULL : (expr_it = (g_object_unref (expr_it), NULL)));
	}
	vala_ccode_writer_write_string (writer, ")");
}


ValaCCodeExpression* vala_ccode_function_call_get_call (ValaCCodeFunctionCall* self) {
	g_return_val_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self), NULL);
	return self->priv->_call;
}


void vala_ccode_function_call_set_call (ValaCCodeFunctionCall* self, ValaCCodeExpression* value) {
	ValaCCodeExpression* _tmp2;
	ValaCCodeExpression* _tmp1;
	g_return_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_call = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_call == NULL ? NULL : (self->priv->_call = (g_object_unref (self->priv->_call), NULL))), _tmp2);
}


static void vala_ccode_function_call_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaCCodeFunctionCall * self;
	self = VALA_CCODE_FUNCTION_CALL (object);
	switch (property_id) {
		case VALA_CCODE_FUNCTION_CALL_CALL:
		g_value_set_object (value, vala_ccode_function_call_get_call (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_function_call_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaCCodeFunctionCall * self;
	self = VALA_CCODE_FUNCTION_CALL (object);
	switch (property_id) {
		case VALA_CCODE_FUNCTION_CALL_CALL:
		vala_ccode_function_call_set_call (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_function_call_class_init (ValaCCodeFunctionCallClass * klass) {
	vala_ccode_function_call_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaCCodeFunctionCallPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_function_call_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_function_call_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_ccode_function_call_dispose;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_function_call_real_write;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_FUNCTION_CALL_CALL, g_param_spec_object ("call", "call", "call", VALA_TYPE_CCODE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_ccode_function_call_init (ValaCCodeFunctionCall * self) {
	self->priv = VALA_CCODE_FUNCTION_CALL_GET_PRIVATE (self);
	self->priv->arguments = GEE_LIST (gee_array_list_new (VALA_TYPE_CCODE_EXPRESSION, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
}


static void vala_ccode_function_call_dispose (GObject * obj) {
	ValaCCodeFunctionCall * self;
	self = VALA_CCODE_FUNCTION_CALL (obj);
	(self->priv->_call == NULL ? NULL : (self->priv->_call = (g_object_unref (self->priv->_call), NULL)));
	(self->priv->arguments == NULL ? NULL : (self->priv->arguments = (g_object_unref (self->priv->arguments), NULL)));
	G_OBJECT_CLASS (vala_ccode_function_call_parent_class)->dispose (obj);
}


GType vala_ccode_function_call_get_type (void) {
	static GType vala_ccode_function_call_type_id = 0;
	if (G_UNLIKELY (vala_ccode_function_call_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeFunctionCallClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_function_call_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeFunctionCall), 0, (GInstanceInitFunc) vala_ccode_function_call_init };
		vala_ccode_function_call_type_id = g_type_register_static (VALA_TYPE_CCODE_EXPRESSION, "ValaCCodeFunctionCall", &g_define_type_info, 0);
	}
	return vala_ccode_function_call_type_id;
}

==> vala-0.3.5.tar.gz: vala-0.3.5/ccode/valaccodefunctioncall.c <==
/* valaccodefunctioncall.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <ccode/valaccodefunctioncall.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <gee/readonlylist.h>
#include <stdlib.h>
#include <string.h>
#include <ccode/valaccodenode.h>




struct _ValaCCodeFunctionCallPrivate {
	ValaCCodeExpression* _call;
	GeeList* arguments;
};

#define VALA_CCODE_FUNCTION_CALL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_FUNCTION_CALL, ValaCCodeFunctionCallPrivate))
enum  {
	VALA_CCODE_FUNCTION_CALL_DUMMY_PROPERTY,
	VALA_CCODE_FUNCTION_CALL_CALL
};
static void vala_ccode_function_call_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static gpointer vala_ccode_function_call_parent_class = NULL;
static void vala_ccode_function_call_finalize (GObject * obj);



ValaCCodeFunctionCall* vala_ccode_function_call_new (ValaCCodeExpression* call) {
	ValaCCodeFunctionCall * self;
	g_return_val_if_fail (call == NULL || VALA_IS_CCODE_EXPRESSION (call), NULL);
	self = g_object_newv (VALA_TYPE_CCODE_FUNCTION_CALL, 0, NULL);
	vala_ccode_function_call_set_call (self, call);
	return self;
}


/**
 * Appends the specified expression to the list of arguments.
 *
 * @param expr a C code expression
 */
void vala_ccode_function_call_add_argument (ValaCCodeFunctionCall* self, ValaCCodeExpression* expr) {
	g_return_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self));
	g_return_if_fail (VALA_IS_CCODE_EXPRESSION (expr));
	gee_collection_add (GEE_COLLECTION (self->priv->arguments), expr);
}


/**
 * Returns a copy of the list of arguments.
 *
 * @return list of arguments
 */
GeeList* vala_ccode_function_call_get_arguments (ValaCCodeFunctionCall* self) {
	g_return_val_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self), NULL);
	return GEE_LIST (gee_read_only_list_new (VALA_TYPE_CCODE_EXPRESSION, ((GBoxedCopyFunc) (g_object_ref)), g_object_unref, self->priv->arguments));
}


static void vala_ccode_function_call_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeFunctionCall * self;
	gboolean first;
	self = VALA_CCODE_FUNCTION_CALL (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->_call), writer);
	vala_ccode_writer_write_string (writer, " (");
	first = TRUE;
	{
		GeeList* expr_collection;
		int expr_it;
		expr_collection = self->priv->arguments;
		for (expr_it = 0; expr_it < gee_collection_get_size (GEE_COLLECTION (expr_collection)); expr_it = expr_it + 1) {
			ValaCCodeExpression* expr;
			expr = ((ValaCCodeExpression*) (gee_list_get (GEE_LIST (expr_collection), expr_it)));
			{
				if (!first) {
					vala_ccode_writer_write_string (writer, ", ");
				} else {
					first = FALSE;
				}
				if (expr != NULL) {
					vala_ccode_node_write (VALA_CCODE_NODE (expr), writer);
				}
				(expr == NULL ? NULL : (expr = (g_object_unref (expr), NULL)));
			}
		}
	}
	vala_ccode_writer_write_string (writer, ")");
}


ValaCCodeExpression* vala_ccode_function_call_get_call (ValaCCodeFunctionCall* self) {
	g_return_val_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self), NULL);
	return self->priv->_call;
}


void vala_ccode_function_call_set_call (ValaCCodeFunctionCall* self, ValaCCodeExpression* value) {
	ValaCCodeExpression* _tmp2;
	ValaCCodeExpression* _tmp1;
	g_return_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_call = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_call == NULL ? NULL : (self->priv->_call = (g_object_unref (self->priv->_call), NULL))), _tmp2);
	g_object_notify (((GObject *) (self)), "call");
}


static void vala_ccode_function_call_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaCCodeFunctionCall * self;
	self = VALA_CCODE_FUNCTION_CALL (object);
	switch (property_id) {
		case VALA_CCODE_FUNCTION_CALL_CALL:
		g_value_set_object (value, vala_ccode_function_call_get_call (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_function_call_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaCCodeFunctionCall * self;
	self = VALA_CCODE_FUNCTION_CALL (object);
	switch (property_id) {
		case VALA_CCODE_FUNCTION_CALL_CALL:
		vala_ccode_function_call_set_call (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_ccode_function_call_class_init (ValaCCodeFunctionCallClass * klass) {
	vala_ccode_function_call_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaCCodeFunctionCallPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_function_call_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_function_call_set_property;
	G_OBJECT_CLASS (klass)->finalize = vala_ccode_function_call_finalize;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_function_call_real_write;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_FUNCTION_CALL_CALL, g_param_spec_object ("call", "call", "call", VALA_TYPE_CCODE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_ccode_function_call_instance_init (ValaCCodeFunctionCall * self) {
	self->priv = VALA_CCODE_FUNCTION_CALL_GET_PRIVATE (self);
	self->priv->arguments = GEE_LIST (gee_array_list_new (VALA_TYPE_CCODE_EXPRESSION, ((GBoxedCopyFunc) (g_object_ref)), g_object_unref, g_direct_equal));
}


static void vala_ccode_function_call_finalize (GObject * obj) {
	ValaCCodeFunctionCall * self;
	self = VALA_CCODE_FUNCTION_CALL (obj);
	(self->priv->_call == NULL ? NULL : (self->priv->_call = (g_object_unref (self->priv->_call), NULL)));
	(self->priv->arguments == NULL ? NULL : (self->priv->arguments = (g_object_unref (self->priv->arguments), NULL)));
	G_OBJECT_CLASS (vala_ccode_function_call_parent_class)->finalize (obj);
}


GType vala_ccode_function_call_get_type (void) {
	static GType vala_ccode_function_call_type_id = 0;
	if (vala_ccode_function_call_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeFunctionCallClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_function_call_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeFunctionCall), 0, (GInstanceInitFunc) vala_ccode_function_call_instance_init };
		vala_ccode_function_call_type_id = g_type_register_static (VALA_TYPE_CCODE_EXPRESSION, "ValaCCodeFunctionCall", &g_define_type_info, 0);
	}
	return vala_ccode_function_call_type_id;
}
